﻿Option Strict On
Imports System
Imports System.Collections
Imports System.Collections.Generic
Imports System.IO
Imports System.Linq
Imports System.Runtime.CompilerServices


Module Module1

   Sub Main()



      Dim myStreamReader = New StreamReader("..\..\..\lorem ipsum.txt")
      Dim strLine As String


      Dim result = From line In myStreamReader.Lines _
                   Where line.IndexOf("lorem", StringComparison.OrdinalIgnoreCase) >= 0 _
                   Distinct

      For Each strLine In result
         Console.WriteLine(strLine)
      Next

      Console.ReadLine()

   End Sub

End Module


Module StreamExtensions


   <Extension()> _
   Function Lines(ByVal rdr As TextReader) As IEnumerable(Of String)
      Return New StreamReaderLineEnumerator(rdr)
   End Function




   Public Class StreamReaderLineEnumerator
      Implements IEnumerable(Of String)
      Implements IEnumerator(Of String)

      Private _rdr As TextReader
      Private _current As String

      Sub New(ByVal rdr As TextReader)
         _rdr = rdr
      End Sub


      ''' <summary>
      '''    Get's an enumerator to read lines from a stream
      ''' </summary>
      ''' <returns>IEnumerator(Of String)</returns>
      ''' <remarks>Can only be called once due to the underlying stream. Not thread safe.</remarks>
      ''' <exception cref="InvalidOperationException">thrown if GetEnumerator is called more than once</exception>
      ''' 
      Public Function GetEnumerator() As IEnumerator(Of String) Implements IEnumerable(Of String).GetEnumerator
         Static iFirstTime As Int32
         If iFirstTime = 0 AndAlso Threading.Interlocked.Increment(iFirstTime) = 1 Then
            Return Me
         Else
            Throw New InvalidOperationException("GetEnumerator can only be called once on a stream")
         End If
      End Function


      Public Function MoveNext() As Boolean Implements IEnumerator.MoveNext
         _current = _rdr.ReadLine()
         Return _current IsNot Nothing
      End Function


      Private Function IEnumerator_GetEnumerator() As IEnumerator Implements IEnumerable.GetEnumerator
         Return GetEnumerator()
      End Function


      Public ReadOnly Property Current() As String Implements IEnumerator(Of String).Current
         Get
            Return _current
         End Get
      End Property

      Private ReadOnly Property IEnumerator_Current() As Object Implements IEnumerator.Current
         Get
            Return Current
         End Get
      End Property


      Private Sub Reset() Implements IEnumerator.Reset
         Throw New NotImplementedException()
      End Sub

      Public Sub Dispose() Implements IDisposable.Dispose
         'not implemented
      End Sub

   End Class



End Module





