﻿Option Strict On : Option Explicit On : Option Infer On : Option Compare Binary

Imports System
Imports System.Collections
Imports System.Collections.Generic



''' <summary>
''' A generic template for an iterator. Use with a MoveNextFunc lambda or delegate.
''' In the MoveNextFunc set the current item and return True if there are more items, otherwise return False.
''' </summary>
''' <typeparam name="T">the type of each item in the iterator</typeparam>
''' <remarks></remarks>
Public Class GenericIterator(Of T)
   Implements IEnumerable(Of T)
   Implements IEnumerator(Of T)

   Public Delegate Function MoveNextFunc(ByRef nextItem As T) As Boolean

   Private _Current As T
   Private _func As MoveNextFunc

   Public Sub New(ByVal func As MoveNextFunc)
      _func = func
   End Sub


   Public Function MoveNext() As Boolean Implements IEnumerator.MoveNext
      Return _func(_Current)
   End Function


   Public Function GetEnumerator() As IEnumerator(Of T) Implements IEnumerable(Of T).GetEnumerator
      Static iBeenCalled As Int32
      If (iBeenCalled = 0) AndAlso _
         (Threading.Interlocked.Increment(iBeenCalled) = 1) Then
         Return Me
      Else
         Return New GenericIterator(Of T)(_func)
      End If
   End Function


   Public ReadOnly Property Current() As T Implements IEnumerator(Of T).Current
      Get
         Return _Current
      End Get
   End Property

   Public Overridable Sub Reset() Implements IEnumerator.Reset
      Throw New NotImplementedException("Iterator cannot be reset")
   End Sub


   Private Function IEnumerator_GetEnumerator() As IEnumerator Implements IEnumerable.GetEnumerator
      Return Me.GetEnumerator
   End Function

   Private ReadOnly Property IEnumerator_Current() As Object Implements IEnumerator.Current
      Get
         Return Me.Current
      End Get
   End Property

   Public Sub Dispose() Implements IDisposable.Dispose
      ' not implemented
   End Sub

End Class